@@ -7,6 +7,7 @@
 
 #include "pixman.h"
 #include <time.h>
+#include <assert.h>
 
 #ifndef FALSE
 #define FALSE 0
@@ -148,6 +149,34 @@ typedef struct bits_image bits_image_t;
 typedef struct circle circle_t;
 typedef struct point point_t;
 
+typedef void (* CompositeFunc) (pixman_op_t,
+				pixman_image_t *, pixman_image_t *, pixman_image_t *,
+				int16_t, int16_t, int16_t, int16_t, int16_t, int16_t,
+				uint16_t, uint16_t);
+
+
+/* These "formats" both have depth 0, so they
+ * will never clash with any real ones
+ */
+#define PIXMAN_null		PIXMAN_FORMAT(0,0,0,0,0,0)
+#define PIXMAN_solid		PIXMAN_FORMAT(0,1,0,0,0,0)
+
+#define NEED_COMPONENT_ALPHA		(1 << 0)
+#define NEED_PIXBUF			(1 << 1)
+#define NEED_SOLID_MASK		        (1 << 2)
+
+typedef struct
+{
+    pixman_op_t			op;
+    pixman_format_code_t	src_format;
+    pixman_format_code_t	mask_format;
+    pixman_format_code_t	dest_format;
+    CompositeFunc		func;
+    uint32_t			flags;
+} FastPathInfo;
+
+extern const FastPathInfo *const c_fast_paths;
+
 /* FIXME - the types and structures below should be give proper names
  */
 
@@ -220,11 +249,10 @@ storeProc64 pixman_storeProcForPicture64_accessors (bits_image_t *);
 void pixman_expand(uint64_t *dst, const uint32_t *src, pixman_format_code_t, int width);
 void pixman_contract(uint32_t *dst, const uint64_t *src, int width);
 
-void pixmanFetchSourcePict(source_image_t *, int x, int y, int width,
+void pixmanFetchGradient (gradient_t *, int x, int y, int width,
                            uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
-void pixmanFetchSourcePict64(source_image_t *, int x, int y, int width,
-                             uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
-
+void _pixman_image_get_scanline_64_generic (pixman_image_t * pict, int x, int y, int width,
+					    uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
 void fbFetchTransformed(bits_image_t *, int x, int y, int width,
                         uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
 void fbStoreExternalAlpha(bits_image_t *, int x, int y, int width,
@@ -241,21 +269,11 @@ void fbFetchExternalAlpha_accessors(bits_image_t *, int x, int y, int width,
                                     uint32_t *buffer, uint32_t *mask,
                                     uint32_t maskBits);
 
-void fbFetchTransformed64(bits_image_t *, int x, int y, int width,
-                          uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
 void fbStoreExternalAlpha64(bits_image_t *, int x, int y, int width,
                             uint64_t *buffer);
-void fbFetchExternalAlpha64(bits_image_t *, int x, int y, int width,
-                            uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
 
-void fbFetchTransformed64_accessors(bits_image_t *, int x, int y, int width,
-                                    uint64_t *buffer, uint64_t *mask,
-                                    uint32_t maskBits);
 void fbStoreExternalAlpha64_accessors(bits_image_t *, int x, int y, int width,
                                       uint64_t *buffer);
-void fbFetchExternalAlpha64_accessors(bits_image_t *, int x, int y, int width,
-                                      uint64_t *buffer, uint64_t *mask,
-                                      uint32_t maskBits);
 
 /* end */
 
@@ -277,12 +295,6 @@ typedef enum
     SOURCE_IMAGE_CLASS_VERTICAL,
 } source_pict_class_t;
 
-typedef source_pict_class_t (* classify_func_t) (pixman_image_t *image,
-						 int             x,
-						 int             y,
-						 int             width,
-						 int             height);
-
 typedef void (*scanStoreProc)(pixman_image_t *, int, int, int, uint32_t *);
 typedef void (*scanFetchProc)(pixman_image_t *, int, int, int, uint32_t *,
 			      uint32_t *, uint32_t);
@@ -293,19 +305,43 @@ source_pict_class_t _pixman_image_classify (pixman_image_t *image,
 					    int             width,
 					    int             height);
 
-scanFetchProc
-_pixman_image_get_fetcher (pixman_image_t *image,
-			   int             wide);
+void
+_pixman_image_get_scanline_32 (pixman_image_t *image, int x, int y, int width,
+			       uint32_t *buffer, uint32_t *mask, uint32_t mask_bits);
+
+/* Even thought the type of buffer is uint32_t *, the function actually expects
+ * a uint64_t *buffer.
+ */
+void
+_pixman_image_get_scanline_64 (pixman_image_t *image, int x, int y, int width,
+			       uint32_t *buffer, uint32_t *unused, uint32_t unused2);
 
 scanStoreProc
 _pixman_image_get_storer (pixman_image_t *image,
 			  int             wide);
 
+pixman_image_t *
+_pixman_image_allocate (void);
+
+pixman_bool_t
+_pixman_init_gradient (gradient_t     *gradient,
+		       const pixman_gradient_stop_t *stops,
+		       int	       n_stops);
+void
+_pixman_image_reset_clip_region (pixman_image_t *image);
+
 struct point
 {
     int16_t x, y;
 };
 
+typedef source_pict_class_t (* classify_func_t) (pixman_image_t *image,
+						 int             x,
+						 int             y,
+						 int             width,
+						 int             height);
+typedef void (* property_changed_func_t)        (pixman_image_t *image);
+
 struct image_common
 {
     image_type_t		type;
@@ -325,6 +361,9 @@ struct image_common
     pixman_read_memory_func_t	read_func;
     pixman_write_memory_func_t	write_func;
     classify_func_t		classify;
+    property_changed_func_t	property_changed;
+    scanFetchProc		get_scanline_32;
+    scanFetchProc		get_scanline_64;
 };
 
 struct source_image
@@ -407,6 +446,39 @@ union pixman_image
     solid_fill_t		solid;
 };
 
+/* Gradient walker
+ */
+typedef struct
+{
+    uint32_t        left_ag;
+    uint32_t        left_rb;
+    uint32_t        right_ag;
+    uint32_t        right_rb;
+    int32_t       left_x;
+    int32_t       right_x;
+    int32_t       stepper;
+
+    pixman_gradient_stop_t	*stops;
+    int                      num_stops;
+    unsigned int             spread;
+
+    int		  need_reset;
+} GradientWalker;
+
+void
+_pixman_gradient_walker_init (GradientWalker  *walker,
+			      gradient_t      *gradient,
+			      unsigned int     spread);
+
+void
+_pixman_gradient_walker_reset (GradientWalker       *walker,
+			       pixman_fixed_32_32_t  pos);
+
+uint32_t
+_pixman_gradient_walker_pixel (GradientWalker       *walker,
+			       pixman_fixed_32_32_t  x);
+
+
 
 #define LOG2_BITMAP_PAD 5
 #define FB_STIP_SHIFT	LOG2_BITMAP_PAD
